@@ -83,6 +83,11 @@ pub(crate) struct Reader {
 
   is_stateful: bool, // is this StatefulReader or Statelessreader as per RTPS spec
   // Currently we support only stateful behaviour.
+  // RTPS Spec: Section 8.4.11.2 Reliable StatelessReader Behavior
+  // "This combination is not supported by the RTPS protocol."
+  // So stateful must be true whenever we are Reliable.
+ 
+  reliability: policy::Reliability, 
   dds_cache: Arc<RwLock<DDSCache>>,
 
   #[cfg(test)]
@@ -123,6 +128,11 @@ impl Reader {
       status_sender: i.status_sender,
       udp_sender,
       is_stateful: true, // Do not change this before stateless functionality is implemented.
+
+      reliability: i
+        .qos_policy
+        .reliability() // use qos specification
+        .unwrap_or(policy::Reliability::BestEffort), // or default to BestEffort
       dds_cache,
       topic_name: i.topic_name,
       qos_policy: i.qos_policy,
@@ -284,7 +294,7 @@ impl Reader {
     let cc = self
       .seqnum_instant_map
       .get(&sequence_number)
-      .and_then(|i| dds_cache.from_topic_get_change(&self.topic_name, i));
+      .and_then(|i| dds_cache.topic_get_change(&self.topic_name, i));
 
     debug!("history cache !!!! {:?}", cc);
 
@@ -299,7 +309,7 @@ impl Reader {
     let cc = self
       .seqnum_instant_map
       .get(&sequence_number)
-      .and_then(|i| dds_cache.from_topic_get_change(&self.topic_name, i));
+      .and_then(|i| dds_cache.topic_get_change(&self.topic_name, i));
     debug!("history cache !!!! {:?}", cc);
     cc.cloned()
   }
@@ -397,6 +407,7 @@ impl Reader {
     remote_group_entity_id: EntityId,
     unicast_locator_list: Vec<Locator>,
     multicast_locator_list: Vec<Locator>,
+    qos: &QosPolicies,
   ) {
     let proxy = RtpsWriterProxy::new(
       remote_writer_guid,
@@ -404,7 +415,7 @@ impl Reader {
       multicast_locator_list,
       remote_group_entity_id,
     );
-    self.update_writer_proxy(proxy, &QosPolicies::qos_none());
+    self.update_writer_proxy(proxy, qos);
   }
 
   fn matched_writer_lookup(&mut self, remote_writer_guid: GUID) -> Option<&mut RtpsWriterProxy> {
@@ -528,16 +539,17 @@ impl Reader {
     writer_sn: SequenceNumber,
   ) {
     trace!(
-      "handle_data_msg from {:?} seq={:?} topic={:?} stateful={:?}",
+      "handle_data_msg from {:?} seq={:?} topic={:?} reliability={:?} stateful={:?}",
       &writer_guid,
       writer_sn,
       self.topic_name,
+      self.reliability,
       self.is_stateful,
     );
     if self.is_stateful {
       let my_entityid = self.my_guid.entity_id; // to please borrow checker
       if let Some(writer_proxy) = self.matched_writer_lookup(writer_guid) {
-        if writer_proxy.contains_change(writer_sn) {
+        if writer_proxy.should_ignore_change(writer_sn) {
           // change already present
           debug!("handle_data_msg already have this seq={:?}", writer_sn);
           if my_entityid == EntityId::SPDP_BUILTIN_PARTICIPANT_READER {
@@ -653,6 +665,7 @@ impl Reader {
 
   // Returns if responding with ACKNACK?
   // TODO: Return value seems to go unused in callers.
+  // ...except in test cases, but not sure if this is strictly necessary to have.
   pub fn handle_heartbeat_msg(
     &mut self,
     heartbeat: &Heartbeat,
@@ -662,11 +675,16 @@ impl Reader {
     let writer_guid =
       GUID::new_with_prefix_and_id(mr_state.source_guid_prefix, heartbeat.writer_id);
 
-    if !self.is_stateful {
+    if self.reliability == policy::Reliability::BestEffort {
       debug!(
-        "HEARTBEAT from {:?}, reader is stateless. Ignoring. topic={:?} reader={:?}",
+        "HEARTBEAT from {:?}, but this Reader is BestEffort. Ignoring. topic={:?} reader={:?}",
         writer_guid, self.topic_name, self.my_guid
       );
+      // BestEffort Reader reacts only to DATA and GAP
+      // See RTPS Spec Section "8.4.11 RTPS StatelessReader Behavior":
+      // Figure 8.23 - Behavior of the Best-Effort StatefulReader with respect to each
+      // matched Writer and
+      // Figure 8.22 - Behavior of the Best-Effort StatelessReader
       return false;
     }
 
@@ -714,7 +732,7 @@ impl Reader {
       };
       for instant in removed_instances.values() {
         if cache
-          .from_topic_remove_change(&self.topic_name, instant)
+          .topic_remove_change(&self.topic_name, instant)
           .is_none()
         {
           debug!("WriterProxy told to remove an instant which was not present");
@@ -759,7 +777,7 @@ impl Reader {
             &missing_seqnums
               .iter()
               .copied()
-              .take_while(|sn| sn < &(first_missing + SequenceNumber::from(256)))
+              .take_while(|sn| sn < &(first_missing + SequenceNumber::new(256)))
               .collect(),
           )
         }
@@ -863,7 +881,7 @@ impl Reader {
       Err(e) => panic!("The DDSCache of is poisoned. Error: {}", e),
     };
     for instant in &removed_changes {
-      cache.from_topic_remove_change(&self.topic_name, instant);
+      cache.topic_remove_change(&self.topic_name, instant);
     }
 
     // Is this needed?
@@ -989,7 +1007,7 @@ impl Reader {
         AckNack {
           reader_id,
           writer_id: writer_proxy.remote_writer_guid.entity_id,
-          reader_sn_state: SequenceNumberSet::new_empty(SequenceNumber::from(1)),
+          reader_sn_state: SequenceNumberSet::new_empty(SequenceNumber::new(1)),
           count: acknack_count,
         },
         InfoDestination {
@@ -1033,15 +1051,16 @@ impl fmt::Debug for Reader {
 
 #[cfg(test)]
 mod tests {
-  use super::*;
   use crate::{
-    dds::{statusevents::DataReaderStatus, typedesc::TypeDesc, with_key::datawriter::WriteOptions},
-    messages::submessages::submessage_elements::serialized_payload::SerializedPayload,
-    structure::{
-      guid::{EntityId, EntityKind, GuidPrefix, GUID},
-      topic_kind::TopicKind,
+    dds::{
+      qos::policy::Reliability, statusevents::DataReaderStatus, typedesc::TypeDesc,
+      with_key::datawriter::WriteOptions,
     },
+    messages::submessages::submessage_elements::serialized_payload::SerializedPayload,
+    structure::guid::{EntityId, EntityKind, GuidPrefix, GUID},
+    Duration, QosPolicyBuilder,
   };
+  use super::*;
 
   #[test]
   #[ignore]
@@ -1056,11 +1075,10 @@ mod tests {
       mio_channel::sync_channel::<ReaderCommand>(10);
 
     let dds_cache = Arc::new(RwLock::new(DDSCache::new()));
-    dds_cache.write().unwrap().add_new_topic(
-      "test".to_string(),
-      TopicKind::NoKey,
-      TypeDesc::new("testi".to_string()),
-    );
+    dds_cache
+      .write()
+      .unwrap()
+      .add_new_topic("test".to_string(), TypeDesc::new("testi".to_string()));
 
     let reader_ing = ReaderIngredients {
       guid,
@@ -1094,6 +1112,7 @@ mod tests {
       EntityId::UNKNOWN,
       mr_state.unicast_reply_locator_list.clone(),
       mr_state.multicast_reply_locator_list.clone(),
+      &QosPolicies::qos_none(),
     );
 
     let reader_id = EntityId::create_custom_entity_id([1, 2, 3], EntityKind::from(111));
@@ -1122,11 +1141,10 @@ mod tests {
       mio_channel::sync_channel::<ReaderCommand>(10);
 
     let dds_cache = Arc::new(RwLock::new(DDSCache::new()));
-    dds_cache.write().unwrap().add_new_topic(
-      "test".to_string(),
-      TopicKind::NoKey,
-      TypeDesc::new("testi".to_string()),
-    );
+    dds_cache
+      .write()
+      .unwrap()
+      .add_new_topic("test".to_string(), TypeDesc::new("testi".to_string()));
 
     let reader_ing = ReaderIngredients {
       guid: new_guid,
@@ -1161,6 +1179,7 @@ mod tests {
       EntityId::UNKNOWN,
       mr_state.unicast_reply_locator_list.clone(),
       mr_state.multicast_reply_locator_list.clone(),
+      &QosPolicies::qos_none(),
     );
 
     let d = Data {
@@ -1178,7 +1197,7 @@ mod tests {
     let ddsdata = DDSData::new(d.serialized_payload.unwrap());
     let cc_built_here = CacheChange::new(writer_guid, d_seqnum, WriteOptions::default(), ddsdata);
 
-    let cc_from_chache = hc_locked.from_topic_get_change(
+    let cc_from_chache = hc_locked.topic_get_change(
       &new_reader.topic_name,
       new_reader.seqnum_instant_map.get(&d_seqnum).unwrap(),
     );
@@ -1189,6 +1208,7 @@ mod tests {
 
   #[test]
   fn rtpsreader_handle_heartbeat() {
+    env_logger::init();
     let new_guid = GUID::dummy_test_guid(EntityKind::READER_NO_KEY_USER_DEFINED);
 
     let (send, _rec) = mio_channel::sync_channel::<()>(100);
@@ -1198,17 +1218,21 @@ mod tests {
       mio_channel::sync_channel::<ReaderCommand>(10);
 
     let dds_cache = Arc::new(RwLock::new(DDSCache::new()));
-    dds_cache.write().unwrap().add_new_topic(
-      "test".to_string(),
-      TopicKind::NoKey,
-      TypeDesc::new("testi".to_string()),
-    );
+    dds_cache
+      .write()
+      .unwrap()
+      .add_new_topic("test".to_string(), TypeDesc::new("testi".to_string()));
+    let reliable_qos = QosPolicyBuilder::new()
+      .reliability(Reliability::Reliable {
+        max_blocking_time: Duration::from_millis(100),
+      })
+      .build();
     let reader_ing = ReaderIngredients {
       guid: new_guid,
       notification_sender: send,
       status_sender,
       topic_name: "test".to_string(),
-      qos_policy: QosPolicies::qos_none(),
+      qos_policy: reliable_qos.clone(),
       data_reader_command_receiver: reader_command_receiver,
     };
     let mut new_reader = Reader::new(
@@ -1238,6 +1262,7 @@ mod tests {
       EntityId::UNKNOWN,
       mr_state.unicast_reply_locator_list.clone(),
       mr_state.multicast_reply_locator_list.clone(),
+      &reliable_qos,
     );
 
     let d = DDSData::new(SerializedPayload::default());
@@ -1246,8 +1271,8 @@ mod tests {
     let hb_new = Heartbeat {
       reader_id: new_reader.entity_id(),
       writer_id,
-      first_sn: SequenceNumber::from(1), // First hearbeat from a new writer
-      last_sn: SequenceNumber::from(0),
+      first_sn: SequenceNumber::new(1), // First hearbeat from a new writer
+      last_sn: SequenceNumber::new(0),
       count: 1,
     };
     assert!(!new_reader.handle_heartbeat_msg(&hb_new, true, mr_state.clone())); // should be false, no ack
@@ -1255,8 +1280,8 @@ mod tests {
     let hb_one = Heartbeat {
       reader_id: new_reader.entity_id(),
       writer_id,
-      first_sn: SequenceNumber::from(1), // Only one in writers cache
-      last_sn: SequenceNumber::from(1),
+      first_sn: SequenceNumber::new(1), // Only one in writers cache
+      last_sn: SequenceNumber::new(1),
       count: 2,
     };
     assert!(new_reader.handle_heartbeat_msg(&hb_one, false, mr_state.clone())); // Should send an ack_nack
@@ -1264,7 +1289,7 @@ mod tests {
     // After ack_nack, will receive the following change
     let change = CacheChange::new(
       new_reader.guid(),
-      SequenceNumber::from(1),
+      SequenceNumber::new(1),
       WriteOptions::default(),
       d.clone(),
     );
@@ -1279,8 +1304,8 @@ mod tests {
     let hb_one2 = Heartbeat {
       reader_id: new_reader.entity_id(),
       writer_id,
-      first_sn: SequenceNumber::from(1), // Only one in writers cache
-      last_sn: SequenceNumber::from(1),
+      first_sn: SequenceNumber::new(1), // Only one in writers cache
+      last_sn: SequenceNumber::new(1),
       count: 2,
     };
     assert!(!new_reader.handle_heartbeat_msg(&hb_one2, false, mr_state.clone())); // No acknack
@@ -1288,8 +1313,8 @@ mod tests {
     let hb_3_1 = Heartbeat {
       reader_id: new_reader.entity_id(),
       writer_id,
-      first_sn: SequenceNumber::from(1), // writer has last 2 in cache
-      last_sn: SequenceNumber::from(3),  // writer has written 3 samples
+      first_sn: SequenceNumber::new(1), // writer has last 2 in cache
+      last_sn: SequenceNumber::new(3),  // writer has written 3 samples
       count: 3,
     };
     assert!(new_reader.handle_heartbeat_msg(&hb_3_1, false, mr_state.clone())); // Should send an ack_nack
@@ -1297,7 +1322,7 @@ mod tests {
     // After ack_nack, will receive the following changes
     let change = CacheChange::new(
       new_reader.guid(),
-      SequenceNumber::from(2),
+      SequenceNumber::new(2),
       WriteOptions::default(),
       d.clone(),
     );
@@ -1310,7 +1335,7 @@ mod tests {
 
     let change = CacheChange::new(
       new_reader.guid(),
-      SequenceNumber::from(3),
+      SequenceNumber::new(3),
       WriteOptions::default(),
       d,
     );
@@ -1324,8 +1349,8 @@ mod tests {
     let hb_none = Heartbeat {
       reader_id: new_reader.entity_id(),
       writer_id,
-      first_sn: SequenceNumber::from(4), // writer has no samples available
-      last_sn: SequenceNumber::from(3),  // writer has written 3 samples
+      first_sn: SequenceNumber::new(4), // writer has no samples available
+      last_sn: SequenceNumber::new(3),  // writer has written 3 samples
       count: 4,
     };
     assert!(new_reader.handle_heartbeat_msg(&hb_none, false, mr_state)); // Should sen acknack
@@ -1348,11 +1373,10 @@ mod tests {
       mio_channel::sync_channel::<ReaderCommand>(10);
 
     let dds_cache = Arc::new(RwLock::new(DDSCache::new()));
-    dds_cache.write().unwrap().add_new_topic(
-      "test".to_string(),
-      TopicKind::NoKey,
-      TypeDesc::new("testi".to_string()),
-    );
+    dds_cache
+      .write()
+      .unwrap()
+      .add_new_topic("test".to_string(), TypeDesc::new("testi".to_string()));
 
     let reader_ing = ReaderIngredients {
       guid: new_guid,
@@ -1388,6 +1412,7 @@ mod tests {
       EntityId::UNKNOWN,
       mr_state.unicast_reply_locator_list.clone(),
       mr_state.multicast_reply_locator_list.clone(),
+      &QosPolicies::qos_none(),
     );
 
     let n: i64 = 10;
@@ -1398,20 +1423,20 @@ mod tests {
     let mut changes = Vec::new();
 
     for i in 0..n {
-      d.writer_sn = SequenceNumber::from(i);
+      d.writer_sn = SequenceNumber::new(i);
       reader.handle_data_msg(d.clone(), BitFlags::<DATA_Flags>::empty(), &mr_state);
       changes.push(reader.history_cache_change(d.writer_sn).unwrap().clone());
     }
 
     // make sequence numbers 1-3 and 5 7 irrelevant
-    let mut gap_list = SequenceNumberSet::new(SequenceNumber::from(4), 7);
-    gap_list.insert(SequenceNumber::from(5));
-    gap_list.insert(SequenceNumber::from(7));
+    let mut gap_list = SequenceNumberSet::new(SequenceNumber::new(4), 7);
+    gap_list.insert(SequenceNumber::new(5));
+    gap_list.insert(SequenceNumber::new(7));
 
     let gap = Gap {
       reader_id: reader.entity_id(),
       writer_id,
-      gap_start: SequenceNumber::from(1),
+      gap_start: SequenceNumber::new(1),
       gap_list,
     };
 
@@ -1419,28 +1444,28 @@ mod tests {
     reader.handle_gap_msg(&gap, &mr_state);
 
     assert_eq!(
-      reader.history_cache_change(SequenceNumber::from(0)),
+      reader.history_cache_change(SequenceNumber::new(0)),
       Some(changes[0].clone())
     );
-    assert_eq!(reader.history_cache_change(SequenceNumber::from(1)), None);
-    assert_eq!(reader.history_cache_change(SequenceNumber::from(2)), None);
-    assert_eq!(reader.history_cache_change(SequenceNumber::from(3)), None);
+    assert_eq!(reader.history_cache_change(SequenceNumber::new(1)), None);
+    assert_eq!(reader.history_cache_change(SequenceNumber::new(2)), None);
+    assert_eq!(reader.history_cache_change(SequenceNumber::new(3)), None);
     assert_eq!(
-      reader.history_cache_change(SequenceNumber::from(4)),
+      reader.history_cache_change(SequenceNumber::new(4)),
       Some(changes[4].clone())
     );
-    assert_eq!(reader.history_cache_change(SequenceNumber::from(5)), None);
+    assert_eq!(reader.history_cache_change(SequenceNumber::new(5)), None);
     assert_eq!(
-      reader.history_cache_change(SequenceNumber::from(6)),
+      reader.history_cache_change(SequenceNumber::new(6)),
       Some(changes[6].clone())
     );
-    assert_eq!(reader.history_cache_change(SequenceNumber::from(7)), None);
+    assert_eq!(reader.history_cache_change(SequenceNumber::new(7)), None);
     assert_eq!(
-      reader.history_cache_change(SequenceNumber::from(8)),
+      reader.history_cache_change(SequenceNumber::new(8)),
       Some(changes[8].clone())
     );
     assert_eq!(
-      reader.history_cache_change(SequenceNumber::from(9)),
+      reader.history_cache_change(SequenceNumber::new(9)),
       Some(changes[9].clone())
     );
   }
