@@ -150,7 +150,7 @@ where
     // Tell dp_event_loop
     self.my_subscriber.remove_reader(self.my_guid);
 
-    // Tell discoery
+    // Tell discovery
     match self
       .discovery_command
       .send(DiscoveryCommand::RemoveLocalReader { guid: self.guid() })
@@ -687,7 +687,7 @@ where
       ),
     };
 
-    let cache_changes = dds_cache.from_topic_get_changes_in_range(
+    let cache_changes = dds_cache.topic_get_changes_in_range(
       &self.my_topic.name(),
       &self.latest_instant,
       &Timestamp::now(),
@@ -1334,7 +1334,8 @@ mod tests {
   use crate::messages::submessages::submessage_flag::*;
 
   #[test]
-  fn dr_get_samples_from_ddschache() {
+  fn dr_get_samples_from_ddscache() {
+    //env_logger::init();
     let dp = DomainParticipant::new(0).expect("Participant creation failed");
     let mut qos = QosPolicies::qos_none();
     qos.history = Some(policy::History::KeepAll);
@@ -1401,6 +1402,7 @@ mod tests {
       EntityId::UNKNOWN,
       mr_state.unicast_reply_locator_list.clone(),
       mr_state.multicast_reply_locator_list.clone(),
+      &QosPolicies::qos_none(),
     );
 
     let data_flags = DATA_Flags::Endianness | DATA_Flags::Data;
@@ -1408,7 +1410,7 @@ mod tests {
     let data = Data {
       reader_id: EntityId::create_custom_entity_id([1, 2, 3], EntityKind::from(111)),
       writer_id: writer_guid.entity_id,
-      writer_sn: SequenceNumber::from(0_i64),
+      writer_sn: SequenceNumber::from(1_i64),
       serialized_payload: Some(SerializedPayload {
         representation_identifier: RepresentationIdentifier::CDR_LE,
         representation_options: [0, 0],
@@ -1419,6 +1421,8 @@ mod tests {
 
     new_reader.handle_data_msg(data, data_flags, &mr_state);
 
+    std::thread::sleep(std::time::Duration::from_millis(100));
+
     matching_datareader.fill_local_datasample_cache();
     let deserialized_random_data = matching_datareader.read(1, ReadCondition::any()).unwrap()[0]
       .value()
@@ -1435,7 +1439,7 @@ mod tests {
     let data2 = Data {
       reader_id: EntityId::create_custom_entity_id([1, 2, 3], EntityKind::from(111)),
       writer_id: writer_guid.entity_id,
-      writer_sn: SequenceNumber::from(1),
+      writer_sn: SequenceNumber::from(2),
       serialized_payload: Some(SerializedPayload {
         representation_identifier: RepresentationIdentifier::CDR_LE,
         representation_options: [0, 0],
@@ -1451,7 +1455,7 @@ mod tests {
     let data3 = Data {
       reader_id: EntityId::create_custom_entity_id([1, 2, 3], EntityKind::from(111)),
       writer_id: writer_guid.entity_id,
-      writer_sn: SequenceNumber::from(2),
+      writer_sn: SequenceNumber::from(3),
       serialized_payload: Some(SerializedPayload {
         representation_identifier: RepresentationIdentifier::CDR_LE,
         representation_options: [0, 0],
@@ -1534,6 +1538,7 @@ mod tests {
       EntityId::UNKNOWN,
       mr_state.unicast_reply_locator_list.clone(),
       mr_state.multicast_reply_locator_list.clone(),
+      &QosPolicies::qos_none(),
     );
 
     // Reader and datareader ready, test with data
