@@ -410,7 +410,7 @@ impl Writer {
               .dds_cache
               .read()
               .unwrap()
-              .from_topic_get_change(&self.my_topic_name, &timestamp)
+              .topic_get_change(&self.my_topic_name, &timestamp)
             {
               partial_message.data_msg(
                 cache_change,
@@ -738,7 +738,7 @@ impl Writer {
           .dds_cache
           .read()
           .unwrap()
-          .from_topic_get_change(&self.my_topic_name, &timestamp)
+          .topic_get_change(&self.my_topic_name, &timestamp)
         {
           // CacheChange found, construct DATA submessage
           partial_message = partial_message.data_msg(
@@ -834,7 +834,7 @@ impl Writer {
         .dds_cache
         .write()
         .unwrap()
-        .from_topic_remove_before(&self.my_topic_name, keep_instant);
+        .topic_remove_before(&self.my_topic_name, keep_instant);
     } else {
       warn!("{:?} missing from instant map", first_keeper);
     }
@@ -939,10 +939,11 @@ impl Writer {
             self.notify_new_data_to_all_readers();
           }
           info!(
-            "Matched new remote reader on topic={:?} reader= {:?}",
+            "Matched new remote reader on topic={:?} reader={:?}",
             self.topic_name(),
-            &reader_proxy
+            &reader_proxy.remote_reader_guid
           );
+          debug!("Reader details: {:?}", &reader_proxy);
         }
       }
       Some(bad_policy_id) => {
@@ -985,12 +986,13 @@ impl Writer {
 
   fn matched_reader_remove(&mut self, guid: GUID) -> Option<RtpsReaderProxy> {
     let removed = self.readers.remove(&guid);
-    if removed.is_some() {
+    if let Some(ref removed_reader) = removed {
       info!(
         "Removed reader proxy. topic={:?} reader={:?}",
         self.topic_name(),
-        removed
+        removed_reader.remote_reader_guid,
       );
+      debug!("Removed reader proxy details: {:?}", removed_reader);
     }
     removed
   }
