@@ -34,7 +34,7 @@ mod tests {
               vendor_id: [0xFF, 0xAA]
           },
           guid_prefix: GuidPrefix {
-              entity_key: [0x01, 0x02, 0x6D, 0x3F,
+              bytes: [0x01, 0x02, 0x6D, 0x3F,
                           0x7E, 0x07, 0x00, 0x00,
                           0x01, 0x00, 0x00, 0x00]
           }
