@@ -136,10 +136,6 @@ impl SerializedPayload {
       value,
     })
   }
-
-  pub fn representation_identifier(&self) -> RepresentationIdentifier {
-    self.representation_identifier
-  }
 }
 
 impl<C: Context> Writable<C> for SerializedPayload {
