@@ -1,7 +1,7 @@
-use crate::structure::{guid::GUID, sequence_number::SequenceNumber, time::Timestamp};
-//use crate::messages::submessages::submessage_elements::serialized_payload::
-// SerializedPayload;
-use crate::dds::ddsdata::DDSData;
+use crate::{
+  dds::{ddsdata::DDSData, with_key::datawriter::WriteOptions},
+  structure::{guid::GUID, sequence_number::SequenceNumber},
+};
 
 #[derive(Debug, PartialOrd, PartialEq, Ord, Eq, Copy, Clone)]
 pub enum ChangeKind {
@@ -14,7 +14,7 @@ pub enum ChangeKind {
 pub struct CacheChange {
   pub writer_guid: GUID,
   pub sequence_number: SequenceNumber,
-  pub source_timestamp: Option<Timestamp>,
+  pub write_options: WriteOptions,
   pub data_value: DDSData,
 }
 
@@ -23,7 +23,7 @@ impl PartialEq for CacheChange {
   fn eq(&self, other: &Self) -> bool {
     self.writer_guid == other.writer_guid
       && self.sequence_number == other.sequence_number
-      && self.source_timestamp == other.source_timestamp
+      && self.write_options == other.write_options
       && self.data_value == other.data_value
   }
 }
@@ -32,13 +32,13 @@ impl CacheChange {
   pub fn new(
     writer_guid: GUID,
     sequence_number: SequenceNumber,
-    source_timestamp: Option<Timestamp>,
+    write_options: WriteOptions,
     data_value: DDSData,
   ) -> CacheChange {
     CacheChange {
       writer_guid,
       sequence_number,
-      source_timestamp,
+      write_options,
       data_value,
     }
   }
